#  Copyright 2019 - Nokia Corporation
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.
from cliff import lister

from vitrageclient.common import utils


class ServiceList(lister.Lister):
    """List all services"""

    def get_parser(self, prog_name):
        parser = super(ServiceList, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        service = utils.get_client(self).service.list()
        return utils.list2cols_with_rename(
            (
                ('Name', 'name'),
                ('Process Id', 'process'),
                ('Hostname', 'hostname'),
                ('Created At', 'created'),
            ), service)
